package explorviz.hpc_monitoring.filter.counting;

public final class CountingThroughputFilter {
    private volatile long firstIntervalStart              = -1;

    private final long    intervalSize;

    private int           currentCountForCurrentInterval  = 0;

    private volatile long firstTimestampInCurrentInterval = -1;
    private volatile long lastTimestampInCurrentInterval  = -1;
    private final String  beforeOut;

    public CountingThroughputFilter(final String beforeOut) {
        this.beforeOut = beforeOut;
        intervalSize = 1 * 1000 * 1000;
    }

    private void processEvent(final Object event, final long currentTime) {
        final long startOfTimestampsInterval = computeFirstTimestampInInterval(currentTime);
        final long endOfTimestampsInterval = computeLastTimestampInInterval(currentTime);

        if (endOfTimestampsInterval > lastTimestampInCurrentInterval) {
            if (firstTimestampInCurrentInterval >= 0) {
                final long currentCount = currentCountForCurrentInterval;
                System.out.println(beforeOut + ": " + currentCount);
            }

            firstTimestampInCurrentInterval = startOfTimestampsInterval;
            lastTimestampInCurrentInterval = endOfTimestampsInterval;
            currentCountForCurrentInterval = 0;
        }

        currentCountForCurrentInterval++;
    }

    public final void inputObjects(final Object object) {
        processEvent(object, System.currentTimeMillis() * 1000);
    }

    private long computeFirstTimestampInInterval(final long timestamp) {
        if (firstIntervalStart == -1) {
            firstIntervalStart = timestamp;
        }

        return ((timestamp / intervalSize) * intervalSize);
    }

    private long computeLastTimestampInInterval(final long timestamp) {
        return ((((timestamp / intervalSize) + 1) * intervalSize) - 1);
    }
}
