package explorviz.hpc_monitoring.reader;

import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

public final class TimeReader {
	private final long period;

	private final ScheduledExecutorService executorService;

	private final IPeriodicTimeSignalReceiver receiver;

	public TimeReader(final long periodInNanoSec,
			final IPeriodicTimeSignalReceiver receiver) {
		period = periodInNanoSec;
		this.receiver = receiver;
		executorService = new ScheduledThreadPoolExecutor(1);
	}

	public void start() {
		executorService.scheduleAtFixedRate(new Runnable() {
			@Override
			public void run() {
				sendTimestampEvent();
			}
		}, 0, period, TimeUnit.NANOSECONDS);
	}

	protected void sendTimestampEvent() {
		final long timestamp = System.nanoTime();
		receiver.periodicTimeSignal(timestamp);
	}
}
